#ifndef FAREDITOR_INLINE_INCLUDED
#define FAREDITOR_INLINE_INCLUDED

#include "exception.h"

#include <sstream>

inline
GetEditorInfo::GetEditorInfo()
{
	ASSERT_FAR(farManager.editorControl(ECTL_GETINFO, this), "Cannot get editor info");
}

inline
EditorPositionSaver::EditorPositionSaver()
: committed_(false)
{
	ASSERT_FAR(farManager.editorControl(ECTL_GETINFO, this), "Cannot get editor info");
}

inline
EditorPositionSaver::~EditorPositionSaver()
{
	if (!committed_) revert();
}

inline
void
EditorPositionSaver::commit()
{
	committed_ = true;
}


inline
void
EditorPositionSaver::revert()
{
	EditorSetPosition esp = {
		CurLine,
		DEFAULT,
		CurTabPos,
		TopScreenLine,
		LeftPos,
		Overtype
	};
	static_cast<void>(farManager.editorControl(ECTL_SETPOSITION, &esp));
}


inline
EditorLine::EditorLine(const EditorGetString& getString)
:	lineNumber_(getString.StringNumber),
	begin_(getString.StringText),
	size_(getString.StringLength),
	eol_(getString.StringEOL),
	selStart_(getString.SelStart),
	selEnd_(getString.SelEnd)
{
}

inline
EditorLine::const_iterator
EditorLine::begin() const
{
	return begin_;
}

inline
EditorLine::const_iterator
EditorLine::end() const
{
	return begin_ + size_;
}

inline
size_t
EditorLine::size() const
{
	return size_;
}

inline
EditorLine::const_iterator
EditorLine::eol() const
{
	return eol_;
}

inline
bool
EditorLine::selected() const
{
	return selStart_ != -1 && (selEnd_ > selStart_ || selEnd_ == -1);
}

inline
void
EditorLine::set(const char* begin, size_t size)
{
	EditorSetString setString = {
		lineNumber_,
		const_cast<char*>(begin),
		const_cast<char*>(eol_),
        size
	};
	ASSERT_FAR(farManager.editorControl(ECTL_SETSTRING, &setString), "Cannot modify line");
}


inline
void
FarEditor::invalidate()
{
	dirty_ = true;
}

inline
void
FarEditor::redraw()
{
	if (dirty_)
	{
		ASSERT_FAR(farManager.editorControl(ECTL_REDRAW, 0), "Cannot redraw editor");
		dirty_ = false;
	}
}

inline
EditorLine
FarEditor::currentLine()
{
	EditorGetString egs = { CURRENT_LINE };
	ASSERT_FAR(farManager.editorControl(ECTL_GETSTRING, &egs), "Cannot get line from editor");
	return EditorLine(egs);
}

// Loops over all lines in the specified range [startLine, endLine).
// Applies the given predicate to each line.
// If predicate returns true, the loop is terminated.
// If predicate returns false for all lines, the previous editor position is restored.
template <typename Predicate>
bool
FarEditor::findLineIf(int startLine, int endLine, Predicate predicate)
{
	EditorPositionSaver info;
	if (startLine == CURRENT_LINE) startLine = info.CurLine;
	if (endLine == END_LINE) endLine = info.TotalLines;

	EditorSetPosition esp =
	{
		startLine,
		DEFAULT,
		info.CurTabPos,
		DEFAULT,
		info.LeftPos,
		DEFAULT
	};

	for (; esp.CurLine < endLine; ++esp.CurLine)
	{
		esp.TopScreenLine = (std::max)(0, esp.CurLine - (info.WindowSizeY - 1) / 2);
		ASSERT_FAR(farManager.editorControl(ECTL_SETPOSITION, &esp),
			"Cannot set editor position");

		if (predicate(currentLine()))
		{
			invalidate();
			info.commit();
			return true;
		}
	}

	invalidate();
	return false;
}

template <typename Functor>
class False
{
public:
	explicit False(Functor functor)
	: functor_(functor)
	{
	}

	bool operator()(EditorLine& line) const
	{
		functor_(line);
		return false;
	}
private:
	Functor functor_;
};

template <typename Functor>
void
FarEditor::forEachLine(int startLine, int endLine, Functor functor)
{
	findLineIf(startLine, endLine, False<Functor>(functor));
}

inline
void
FarEditor::convertToOem(char* text, size_t size)
{
	EditorConvertText cvt = {
		text,
		size
	};
	ASSERT_FAR(farManager.editorControl(ECTL_EDITORTOOEM, &cvt), "Cannot convert to OEM");
}

inline
void
FarEditor::convertFromOem(char* text, size_t size)
{
	EditorConvertText cvt = {
		text,
		size
	};
	ASSERT_FAR(farManager.editorControl(ECTL_OEMTOEDITOR, &cvt), "Cannot convert from OEM");
}

inline
void
FarEditor::setPosition(int line, int pos, int tabPos,
	int topLine, int leftPos, int overtype)
{
	EditorSetPosition esp = {
		line,
		pos,
		tabPos,
		topLine,
		leftPos,
		overtype
	};
	ASSERT_FAR(farManager.editorControl(ECTL_SETPOSITION, &esp), "Cannot set editor position");
}

inline
void
FarEditor::insertLine()
{
	ASSERT_FAR(farManager.editorControl(ECTL_INSERTSTRING, 0), "Cannot insert line");
}

inline
void
FarEditor::insertText(const char* text)
{
	ASSERT_FAR(farManager.editorControl(ECTL_INSERTTEXT, const_cast<char*>(text)), "Cannot insert text");
}

inline
void
FarEditor::deleteLine()
{
	ASSERT_FAR(farManager.editorControl(ECTL_DELETESTRING, 0), "Cannot delete line");
}

inline
void
FarEditor::save()
{
	ASSERT_FAR(farManager.editorControl(ECTL_SAVEFILE, 0), "Cannot save file");
}

#endif
